"""
This module contains everything related to the game.
"""

from time import sleep
from enum import Enum

from tron.map import Map

class Winner(Enum):
    PLAYER_ONE = 1
    PLAYER_TWO = 2

class Tile(Enum):
    """
    The different type of elements that can be in a map.
    """
    EMPTY = 0
    WALL = -1
    PLAYER_ONE_BODY = 1
    PLAYER_ONE_HEAD = 2
    PLAYER_TWO_BODY = 3
    PLAYER_TWO_HEAD = 4

    def color(self):
        """
        Converts an element of a map to a nice color.
        """
        if self == Tile.EMPTY:
            return (255, 255, 255)
        elif self == Tile.WALL:
            return (0, 0, 0)
        elif self == Tile.PLAYER_ONE_BODY:
            return (128, 0, 0)
        elif self == Tile.PLAYER_ONE_HEAD:
            return (255, 0, 0)
        elif self == Tile.PLAYER_TWO_BODY:
            return (0, 128, 0)
        elif self == Tile.PLAYER_TWO_HEAD:
            return (0, 255, 0)
        else:
            return None

class PositionPlayer:
    """
    A container to store a player with is id, position and boolean indicating
    whether it's still alive or not.
    """
    def __init__(self, id, player, position):
        self.id = id
        self.player = player
        self.position = position
        self.alive = True

    def body(self):
        """
        Returns the body type of the PP depending on its id.
        """
        if self.id == 1:
            return Tile.PLAYER_ONE_BODY
        elif self.id == 2:
            return Tile.PLAYER_TWO_BODY

    def head(self):
        """
        Returns the head type of the PP depending of its id.
        """
        if self.id == 1:
            return Tile.PLAYER_ONE_HEAD
        elif self.id == 2:
            return Tile.PLAYER_TWO_HEAD

class HistoryElement:
    """
    An element from an history.

    It contains the map, but also the direction of each player during the frame.
    The directions of the players will be None at the first frame.
    """
    def __init__(self, mmap, player_one_direction, player_two_direction):
        self.map = mmap
        self.player_one_direction = player_one_direction
        self.player_two_direction = player_two_direction


class Game:
    """
    This class contains the map of the game, and the players.

    It allows to update the player depending on their strategies, and run the game.
    """

    def __init__(self, width, height, pps):
        """
        Returns a new game from its width, height, and number of players.

        Width and height are the number of blocs available in the tron map.
        """
        self.width = width
        self.height = height
        mmap = Map(width, height, Tile.EMPTY, Tile.WALL)
        self.history = [HistoryElement(mmap, None, None)]
        self.pps = pps
        self.winner = None

        for pp in self.pps:
            self.map()[pp.position[0], pp.position[1]] = pp.head()

    def map(self):
        """
        Returns the current map, the last map in the history.
        """
        return self.history[-1].map

    def next_frame(self, window = None):
        """
        Computes the next frame of the game.

        If a window is passed as parameter, events are polled and passed to the
        players, so you can play the game.
        Then, all the players are updated and if a player lands on a square
        that is already occupied or that is outside of the map, the players
        dies.
        """

        # Set previous heads to body
        for pp in self.pps:
            self.map()[pp.position[0], pp.position[1]] = pp.body()

        # Manage the events
        if window:
            import pygame
            while True:
                event = pygame.event.poll()

                if event.type == pygame.NOEVENT:
                    break

                for pp in self.pps:
                    pp.player.manage_event(event)

        player_directions = []

        for pp in self.pps:

            (pp.position, direction) = pp.player.next_position_and_direction(pp.position, self)
            player_directions.append(direction)

            # Check boundaries
            if pp.position[0] < 0 or pp.position[1] < 0 or \
               pp.position[0] >= self.width or pp.position[1] >= self.height:

                pp.alive = False

            # Check collision
            elif self.map()[pp.position[0], pp.position[1]] is not Tile.EMPTY:
                pp.alive = False

            else:
                self.map()[pp.position[0], pp.position[1]] = pp.head()

        # Append to history
        map_clone = self.map().clone()
        self.history.append(HistoryElement(map_clone, player_directions[0], player_directions[1]))


    def main_loop(self, window = None):
        """
        Loops until the game is finished

        If a window is passed as parameter, the game is rendered on the window.
        """

        if window:
            window.render_map(self.map())

        while True:
            alive_count = 0
            alive = None

            if window:
                sleep(0.1)

            for pp in self.pps:
                if pp.alive:
                    pp.player.direction = pp.player.action(self) or pp.direction

            self.next_frame(window)

            for pp in self.pps:
                if pp.alive:
                    alive_count += 1
                    alive = pp.id

            if alive_count <= 1:
                if alive_count == 1:
                   # If player one and two share the same position, it will
                   # mean that they reached the same tile at the same
                   # moment, so it's really a draw.
                    if self.pps[0].position[0] != self.pps[1].position[0] or \
                       self.pps[0].position[1] != self.pps[1].position[1]:

                       # Otherwise, the winner is the player that is still alive.
                       self.winner = alive
                break

            if window:
                window.render_map(self.map())

